#=======================================================================
#	$Id: Login.pm,v 1.4 2011/01/18 15:42:04 pythontech Exp $
#	Login using session cookie
#	Copyright (C) 2005-2011  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::Login;
use strict;
use PythonTech::Conv qw(hq);

#-----------------------------------------------------------------------
#	Configuration parameters:
#	  userDb	Database of users
#			e.g. PythonTech::UserDB::HtPasswd instance
#			or HASH {username => crypted-password}
#	  templater	(optional) Template-filler
#			e.g. PythonTech::Template instance
#	  loginTemplate	(optional) name of template file with fields
#			'username', 'error', 'selfurl'
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {@_};
    foreach (qw(userDb)) {
	die "$class: $_ not configured\n" unless $self->{$_};
    }
    bless $self, $class;
    return $self;
}

sub webquery {
    my($self, $query) = @_;
    my $response = $query->response;
    my $session = $query->session; # Find existing session or start new one
    my $method = $query->request_method;
    my $logged_in;
    my($username,$password,$error);
    my $afterLogin = ($query->param("afterLogin") ||
		      $query->param("after-login") ||
		      '');
    if ($method eq 'POST') {
	# Attempt to log in
	$username = $query->param("username");
	$password = $query->param("password");
	if ($username eq '') {
	    $error = "Please enter your username";
	} elsif ($password eq '') {
	    $error = "Please enter your password";
	} else {
	    # User has supplied username & password - check them
	    my $userdb = $self->{'userDb'};
	    my($validated,$verr);
	    if (ref($userdb) eq 'HASH') {
		# Simple lookup of crypted password
		my $crypted = $userdb->{$username};
		if (defined $crypted &&
		    crypt($password, substr($crypted,0,2)) eq $crypted) {
		    $validated = 1;
		}
	    } elsif (ref($userdb) =~ /[^A-Z]/) {
		# Blessed object
		$userdb->can('validate_user') or
		    die "userDb has no validate_user method";
		if (eval {$self->{'userDb'}->validate_user($username,
							   $password)}) {
		    $validated = 1;
		} else {
		    $verr = $@;
		}
	    } else {
		die "userDb is neither blessed ref nor HASH";
	    }

	    if ($validated) {
		$query->login($username);
		$logged_in = 1;
	    } else {
		$error = "Wrong username or password".($verr ? " ($verr)":"").
		    " - try again";
	    }
	}
    } elsif ($method ne 'GET' && $method ne 'HEAD') {
	die "Unknown request method '$method'";
    }

    if ($logged_in) {
	# Where to go next - either from form or from session
	if ($afterLogin eq '') {
	    $afterLogin = $session->get("afterLogin");
	}
	if ($afterLogin ne '') {
	    $session->clear(['afterLogin']);
	    $response->see_other($afterLogin);
	} else {
	    my $username = $session->get('userName');
	    $response->set_header('X-ptweb-login', 'OK');
	    $response->write("<title>Logged in</title>\n",
			     "You have successfully logged in as ",
			     hq($username), "<br>\n");
	}
    } else {
	# Print login page
	if ($error) {
	    $response->set_header('X-ptweb-login', 'FAIL');
	}
	my $html = $self->login_page($query, $username, $error, $afterLogin);
	$response->write($html);
    }
}

sub login_page {
    my($self, $query,$username,$error,$afterLogin) = @_;
    my $selfurl = $query->url(-path_info=>1);
    my $html;
    if ($self->{'templater'} && $self->{'loginTemplate'}) {
	my $data = {
	    username => $username,
	    error => $error,
	    selfurl => $selfurl,
	    afterLogin => $afterLogin,
	};
	$html = $self->{'templater'}->process_file($self->{'loginTemplate'},
						   $data);
    } else {
	$html .= ("<html>\n".
		  "<head><title>Login</title></head>\n".
		  "<body>\n".
		  "Please log in<br />\n".
		  "<form method=\"POST\" action=\"".hq($selfurl)."\">\n");
	if ($error) {
	    $html .= "<p style=\"color:red\">".hq($error)."</p>\n";
	}
	$html .= ("Name: ".
		  "<input type=\"text\" name=\"username\" value=\"".
		  hq($username)."\" /><br />\n".
		  "Password: ".
		  "<input type=\"password\" name=\"password\" value=\"\" />\n".
		  "<input type=\"hidden\" name=\"afterLogin\" value=\"".
		  hq($afterLogin)."\" />\n".
		  "<input type=\"submit\" value=\"Log in\" />\n".
 #   "tries = ", $session->param("~login-trials"), "\n",
		  "</form>\n".
		  "</body>\n".
		  "</html>\n");
    }
    return $html;
}

1;
